%% Az algoritmusokat segt, hozzjuk szksges grfkezel, grftranszforml fggvnyek.
%% Mivel ezek a fggvnyek nmagukban is rtelmesek, hasznlhatak, ezrt egy kln modulba kerltek.

-module(common).

-export([contains/2]).
-export([init_vertex_labels/2, init_edge_labels/2]).
-export([has_negative_edge/1, has_directed_edge/1]).
-export([search_min_vertex/1, get_vertex_labels/1]).

%% Rekordok.
%% A vertex_label egy cscs cmkje, amely lerja az llapott egy algoritmus aktulis fzisban. Az algoritmusok s a segdfggvnyek hasznljk.
-include("record_vertex_label.hrl").
%% A vertex_info egy cscsrl egy algoritmus elvgzse utn informcit nyjt rekord. Tbb algoritmus kimenete ilyen rekordok listja.
-include("record_vertex_info.hrl").

%% Listakezel fggvny, amely megadja, hogy egy lista (1. paramter) tartalmaz-e egy adott elemet (2. paramter).
contains([], _Elem) ->
    false;

contains([CElem | List], Elem) ->
    if
	CElem =:= Elem -> true;
	true -> contains(List, Elem)
    end.

%% Inicializlja egy grf cscscmkit. A cmkk hrom rtkkel rendelkeznek rekord formjban:
%% - depth: a cscs mlysge az algoritmus bejrsa folyamn. (Kiindulsi rtk: infinite, a Start cscsra: 0.)
%% - parent: a szl cscs, ahonnan az algoritmus folyamn eljutottunk a jelenlegi cscsba. (Kiindulsi rtk: none, a Start cscsra: root.)
%% - done: a cscs ksz van-e az algoritmus szmra? (Kiindulsi rtk: false.)
%% Nem minden algoritmus hasznlja a cscscmkk mind a hrom mezjt.
%% Bemen paramterknt a grf s a Start cscs adand meg.
%% A fggvny visszaadja a megcmkzett grfot. A grf rgi cscscmki trlsre kerlnek.
init_vertex_labels(Graph, Node) ->
    Vertices = digraph:vertices(Graph),
    % Miden cscshoz hozzadunk egy j vertex_label rekordot.
    lists:foreach(
      fun(CNode) ->
	      if
		  % A Start cscsnl ms rtkeket kell belltani.
		  Node == CNode ->
		      Value = #vertex_label{depth = 0, parent = root};
		  true ->
		      Value = #vertex_label{}
	      end,
	      digraph:add_vertex(Graph, CNode, Value)
      end,
      Vertices),
    Graph.

%% Inicializlja egy grf lcmkit. Amennyiben egy lcmke nem szm, egy alaprtelmezett rtkre cserli.
%% Bemen paramterknt a grf s az alaprtelmezett rtk adand meg.
%% A fggvny visszatrsi rtke egy olyan grf, ahol minden lcmke bizonyosan szm.
init_edge_labels(Graph, Default) ->
    Edges = digraph:edges(Graph),
    % Minden let megvizsglunk s ha nem szm a cmkje, akkorm javtjuk az alaprtelmezett rtkkel.
    lists:foreach(
      fun(CEdge) ->
	      {Edge, From, To, Value} = digraph:edge(Graph, CEdge),
	      Numeric = is_number(Value),
	      if
		  not Numeric ->
		      digraph:add_edge(Graph, Edge, From, To, Default);
		  true ->
		      ok
	      end
      end,
      Edges),
    Graph.

%% A fggvny megadja, hogy egy grf tartalmaz-e negatv lcmkt.
has_negative_edge(Graph) ->
    Edges = digraph:edges(Graph),
    has_negative_edge(Graph, Edges).

has_negative_edge(_Graph, []) ->
    false;

has_negative_edge(Graph, [Edge | Edges]) ->
    % Az leket az els olyanig vizsgljuk, amelynek negatv a cscscmkje.
    {_CEdge, _CFrom, _CTo, CValue} = digraph:edge(Graph, Edge),
    if
	CValue < 0 ->
	    true;
	true ->
	    has_negative_edge(Graph, Edges)
    end.

%% A fggvny megadja, hogy egy grfban van-e irnytott l.
%% Az Erlang digraph modulja csak irnytott leket tmogat, ezrt egy l akkor minsl irnytatlannak, ha kt cscs kztt mindkt irnyban ltezik azonos cmkvel.
has_directed_edge(Graph) ->
    Edges = digraph:edges(Graph),
    % A grf leit kitranszformljuk egy hrmas tuplbe: {kiindulsi cscs, cl cscs, lcmke}.
    EdgeDescs = lists:map(
      fun(Edge) ->
	      {_Edge, From, To, Value} = digraph:edge(Graph, Edge),
	      {From, To, Value}
      end,
		  Edges),
    has_directed_edge(Graph, EdgeDescs).

has_directed_edge(_Graph, []) ->
    false;

has_directed_edge(Graph, [{From, To, Value} | EdgeDescs]) ->
    % Minden lhez megkeressk az ellenkez irny prjt s mindkettt kivesszk a listbl a rekurzv hvs eltt.
    % Ha egy lnek nincs azonos cmkj, ellenttes irny prja, akkor talltunk egy irnytott let, nincs szksg tovbbi vizsglatra.
    Contains = contains(EdgeDescs, {To, From, Value}),
    if
	Contains ->
	    has_directed_edge(Graph, lists:delete({To, From, Value}, EdgeDescs));
	true ->
	    true
    end.

%% Megkeresi egy grfban a minimlis mlysgi rtkkel rendelkez, mg nem ksz cscsot.
%% A fggvny visszatrsi rtke egy vertex_info rekord (node, depth, parent) a minimlis mlysgi rtk cscsrl.
%% Amennyiben a grfban mr nincsen olyan nem ksz cscs, amelynek mlysgi rtke nem infinite, akkor egy {node = null, depth = infinite, parent = none} rekorddal tr vissza a fggvny.
search_min_vertex(Graph) ->
    search_min_vertex(Graph, digraph:vertices(Graph), {null, #vertex_label{}}).

search_min_vertex(_Graph, [], {Node, #vertex_label{depth = Depth, parent = Parent}}) ->
    #vertex_info{node = Node, depth = Depth, parent = Parent};

% A fggvny bels megvalstsban hrom paramterrel rendelkezik:
% - a grf,
% - a mg vizsglatra szorul cscsok listja,
% - az eddigi minimlis mlysg cscs.
search_min_vertex(Graph, [Node | Nodes], {MNode, MLabel}) ->
    % A fggvny egy lpsben megvizsglja az elst a htralv cscsok listjbl s ha az jobb az eddigi "legjobb"-nl, akkor a rekurzv hvsban ezt adja t "legjobb"-knt.
    {CNode, CLabel} = digraph:vertex(Graph, Node),
    if
	CLabel#vertex_label.done == false andalso
	CLabel#vertex_label.depth /= infinite andalso
	(CLabel#vertex_label.depth < MLabel#vertex_label.depth orelse MLabel#vertex_label.depth == infinite) ->
	    search_min_vertex(Graph, Nodes, {CNode, CLabel});
	true ->
	    search_min_vertex(Graph, Nodes, {MNode, MLabel})
    end.

%% A fggvny kitranszformlja egy grf cscsait azokat ler vertex_info rekordok listjv.
%% A vertex_info rekord mezi:
%% - node: a cscs azonostja.
%% - depth: a cscs mlysge az algoritmus bejrsa vgeztvel.
%% - parent: a szl cscs, ahonnan az algoritmus folyamn eljutottunk a jelenlegi cscsba.
get_vertex_labels(Graph) ->
    Vertices = digraph:vertices(Graph),
    lists:map(
      fun(Node) ->
	      {CNode, #vertex_label{depth = CDepth, parent = CParent}} = digraph:vertex(Graph, Node),
	      #vertex_info{node = CNode, depth = CDepth, parent = CParent}
      end,
      Vertices).
